@@ -295,23 +295,24 @@ def PygtailLogger(
 
     Truncates lines at 4KB in case writer goes haywire.
     """
-    pygtail = Pygtail(filename, full_lines=True)
-    pygtail_ok = True
+    pygtail = None
+    if logger.getEffectiveLevel() <= VERBOSE_LEVEL:
+        pygtail = Pygtail(filename, full_lines=True)
 
     def poll() -> None:
-        nonlocal pygtail_ok
-        if pygtail_ok:
+        nonlocal pygtail
+        if pygtail:
             try:
                 for line in pygtail:
                     logger.verbose((prefix + line.rstrip())[:4096])  # pyre-ignore
             except:
-                pygtail_ok = False
                 # cf. https://github.com/bgreenlee/pygtail/issues/48
                 logger.verbose(  # pyre-ignore
                     "incomplete log stream due to the following exception; see %s",
                     filename,
                     exc_info=sys.exc_info(),
                 )
+                pygtail = None
 
     try:
         yield poll
@@ -644,7 +645,7 @@ class FlockHolder(AbstractContextManager):
     def flock(  # pyre-fixme
         self,
         filename: str,
-        mode: str = "rb",
+        mode: str = "",
         exclusive: bool = False,
         wait: bool = False,
         update_atime: bool = False,
@@ -655,7 +656,7 @@ class FlockHolder(AbstractContextManager):
         taken care of).
 
         :param filename: file to open & lock
-        :param mode: open() mode
+        :param mode: open() mode, default: "r+b" if exclusive else "rb"
         :param exclusive: True to open an exclusive lock (default: shared lock)p
         :param wait: True to wait as long as needed to obtain the lock, otherwise (default) raise
                      OSError if the lock isn't available immediately. Self-deadlock is possible;
@@ -676,7 +677,7 @@ class FlockHolder(AbstractContextManager):
                         )
                     )
                     return self._flocks[realfilename][0]
-                openfile = open(realfilename, mode)
+                openfile = open(realfilename, mode or ("r+b" if exclusive else "rb"))
                 try:
                     op = fcntl.LOCK_EX if exclusive else fcntl.LOCK_SH
                     if not wait:
