@@ -19,6 +19,7 @@ import re
 import socket
 from abc import ABC, abstractmethod
 from typing import Tuple, List, Dict, Optional, Callable, Iterable, Set, Any
+from contextlib import ExitStack
 import docker
 from .. import Error, Type, Env, Value, StdLib, Tree, _util
 from .._util import (
@@ -454,13 +455,14 @@ class SwarmContainer(TaskContainer):
             # stream stderr into log
             with PygtailLogger(logger, os.path.join(self.host_dir, "stderr.txt")) as poll_stderr:
                 # poll for container exit
-                running = False
                 while exit_code is None:
-                    time.sleep(random.uniform(1.0, 2.0))  # spread out work over the GIL
+                    # spread out work over the GIL
+                    # TODO: adaptive interval before container starts running (poll less frequently
+                    # if it's already been waiting a long time)
+                    time.sleep(random.uniform(1.0, 2.0))
                     if terminating():
                         raise Terminated() from None
                     if "running" in self._observed_states:
-                        running = True
                         poll_stderr()
                     exit_code = self.poll_service(logger, svc)
                 logger.debug(
@@ -675,13 +677,14 @@ def run_local_task(
 
     # provision run directory and log file
     run_id = run_id or task.name
-    run_id_stack = (_run_id_stack or []) + [run_id]
+    _run_id_stack = _run_id_stack or []
     run_dir = provision_run_dir(task.name, run_dir)
-    write_values_json(inputs, os.path.join(run_dir, "inputs.json"))
 
     logger_prefix = (logger_prefix or ["wdl"]) + ["t:" + run_id]
     logger = logging.getLogger(".".join(logger_prefix))
-    with LoggingFileHandler(logger, os.path.join(run_dir, "task.log")) as fh:
+    logfile = os.path.join(run_dir, "task.log")
+    with ExitStack() as cleanup:
+        fh = cleanup.enter_context(LoggingFileHandler(logger, logfile))  # pylint: disable=no-member
         fh.setFormatter(logging.Formatter(LOGGING_FORMAT))
         logger.notice(  # pyre-fixme
             _(
@@ -694,16 +697,23 @@ def run_local_task(
             )
         )
         logger.info(_("thread", ident=threading.get_ident()))
+        write_values_json(inputs, os.path.join(run_dir, "inputs.json"))
+
+        if not _run_id_stack:
+            assert not _cache
+            cache = cleanup.enter_context(CallCache(cfg, logger))  # pylint: disable=no-member
+            cache.flock(logfile, exclusive=True)  # no containing workflow; flock task.log
+        else:
+            cache = _cache
+        assert cache
 
         try:
-            cache = _cache or CallCache(cfg, logger)
-
             # start plugin coroutines and process inputs through them
             with compose_coroutines(
                 [
                     (
                         lambda kwargs, cor=cor: cor(
-                            cfg, logger, run_id_stack, run_dir, task, **kwargs
+                            cfg, logger, _run_id_stack + [run_id], run_dir, task, **kwargs
                         )
                     )
                     for cor in (
@@ -808,35 +818,30 @@ def _download_input_files(
     cached_hits = 0
     cached_bytes = 0
 
-    def map_files(v: Value.Base) -> Value.Base:
+    def rewriter(uri: str) -> str:
         nonlocal downloads, download_bytes, cached_hits, cached_bytes
-        if isinstance(v, Value.File):
-            if downloadable(cfg, v.value):
-                logger.info(_("download input file", uri=v.value))
-                cached, filename = download(
-                    cfg,
-                    logger,
-                    cache,
-                    v.value,
-                    run_dir=os.path.join(run_dir, "download", str(downloads), "."),
-                    logger_prefix=logger_prefix + [f"download{downloads}"],
-                )
-                v.value = filename
-                sz = os.path.getsize(v.value)
-                if cached:
-                    cached_hits += 1
-                    cached_bytes += sz
-                else:
-                    logger.info(_("downloaded input file", uri=v.value, file=v.value, bytes=sz))
-                    downloads += 1
-                    download_bytes += sz
-        for ch in v.children:
-            map_files(ch)
-        return v
+        if downloadable(cfg, uri):
+            logger.info(_("download input file", uri=uri))
+            cached, filename = download(
+                cfg,
+                logger,
+                cache,
+                uri,
+                run_dir=os.path.join(run_dir, "download", str(downloads), "."),
+                logger_prefix=logger_prefix + [f"download{downloads}"],
+            )
+            sz = os.path.getsize(filename)
+            if cached:
+                cached_hits += 1
+                cached_bytes += sz
+            else:
+                logger.info(_("downloaded input file", uri=uri, file=filename, bytes=sz))
+                downloads += 1
+                download_bytes += sz
+            return filename
+        return uri
 
-    ans = inputs.map(
-        lambda binding: Env.Binding(binding.name, map_files(copy.deepcopy(binding.value)))
-    )
+    ans = Value.rewrite_env_files(inputs, rewriter)
     if downloads or cached_hits:
         logger.notice(  # pyre-fixme
             _(
@@ -1126,6 +1131,9 @@ def link_outputs(
                 assert os.path.isfile(hardlink)
                 newlink = os.path.join(dn, os.path.basename(v.value))
                 os.makedirs(dn, exist_ok=False)
+                if not hardlinks and path_really_within(hardlink, run_dir):
+                    # make symlink relative
+                    hardlink = os.path.relpath(hardlink, start=os.path.realpath(dn))
                 (os.link if hardlinks else os.symlink)(hardlink, newlink)
                 v.value = newlink
         # recurse into compound values
